package cn.estsh.i3plus.ext.mes.pcn.apiservice.controller.busi;


import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesSortShippingCheckService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.util.MesRedisLockUtil;
import cn.estsh.i3plus.ext.mes.pcn.pojo.constant.MesCommonConstant;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesSortShippingCheckModel;
import cn.estsh.i3plus.pojo.mes.bean.MesShippingOrderManagement;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

@RestController
@Api(tags = "排序发运校验")
@RequestMapping(MesCommonConstant.MES_YANFEN + "/sort-shipping-check")
@Slf4j
public class MesSortShippingCheckController {

    @Autowired
    private IMesSortShippingCheckService sortShippingCheckService;

    @Autowired
    private MesRedisLockUtil redisLockUtil;

    @GetMapping("/query-loading-order")
    @ApiOperation(value = "查询发运单")
    public ResultBean queryLoadingOrderNo(MesShippingOrderManagement shippingOrderManagement) {

        String moduleKey = "sortShippingCheck:loadingOrderNo:" + shippingOrderManagement.getShippingOrderNo();

        //加锁
        boolean lockFlg = false;
        try {

            //加锁
            lockFlg = redisLockUtil.redisLock(moduleKey);

            // 数据校验
            ValidatorBean.checkNotNull(shippingOrderManagement.getShippingOrderNo(), "发运单不能为空");
            shippingOrderManagement.setOrganizeCode(AuthUtil.getOrganizeCode());

            MesSortShippingCheckModel model = sortShippingCheckService.queryShippingOrderNo(shippingOrderManagement);
            return ResultBean.success("查询成功").setResultObject(model);
        } catch (ImppBusiException imppException) {
            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        } finally {
            //解锁
            redisLockUtil.redisUnLock(lockFlg, moduleKey);
        }
    }

    @PostMapping("/scan-sn")
    @ApiOperation(value = "扫描条码")
    public ResultBean saveSn(@RequestBody MesSortShippingCheckModel model) {

        String moduleKey = "sortShippingCheck:sn:" + model.getSn();

        //加锁
        boolean lockFlg = false;
        try {

            //加锁
            lockFlg = redisLockUtil.redisLock(moduleKey);

            // 数据校验
            ValidatorBean.checkNotNull(model.getShippingOrderNo(), "发运单不能为空");
            ValidatorBean.checkNotNull(model.getDetailList(), "发运明细不能为空");
            ValidatorBean.checkNotNull(model.getSn(), "条码不能为空");

            sortShippingCheckService.saveSn(model, AuthUtil.getOrganizeCode());
            return ResultBean.success("查询成功").setResultObject(model);
        } catch (ImppBusiException imppException) {
            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        } finally {
            //解锁
            redisLockUtil.redisUnLock(lockFlg, moduleKey);
        }
    }

    @PostMapping("/scan-location-code")
    @ApiOperation(value = "扫描位置码")
    public ResultBean saveLocationCode(@RequestBody MesSortShippingCheckModel model) {

        String moduleKey = "sortShippingCheck:sn:" + model.getSn();

        //加锁
        boolean lockFlg = false;
        try {

            //加锁
            lockFlg = redisLockUtil.redisLock(moduleKey);

            // 数据校验
            ValidatorBean.checkNotNull(model.getShippingOrderNo(), "发运单不能为空");
            ValidatorBean.checkNotNull(model.getDetailList(), "发运明细不能为空");
            ValidatorBean.checkNotNull(model.getSn(), "条码不能为空");
            ValidatorBean.checkNotNull(model.getLocationCode(), "位置码不能为空");

            sortShippingCheckService.saveSnAndLocationCode(model, AuthUtil.getOrganizeCode());
            return ResultBean.success("查询成功").setResultObject(model);
        } catch (ImppBusiException imppException) {
            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        } finally {
            //解锁
            redisLockUtil.redisUnLock(lockFlg, moduleKey);
        }
    }


    @GetMapping("/query-auto-push-order")
    @ApiOperation(value = "自动推单")
    public ResultBean queryAutoPushOrder(String shippingGroupNo) {

        try {

            MesShippingOrderManagement order = sortShippingCheckService.queryShippingOrderNoByAutoPushOrder(shippingGroupNo, AuthUtil.getOrganizeCode());
            return ResultBean.success("查询成功").setResultObject(order);
        } catch (ImppBusiException imppException) {
            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

}
