package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesJisShippingService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesShippingLoadingCheckService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.constant.MesCommonConstant;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesShippingLoadingCheckModel;
import cn.estsh.i3plus.platform.common.util.MesPcnConstWords;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.repository.*;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

/**
 * @Description :
 * @Reference :
 * @Author :
 * @CreateDate : 2024-04-26 15:38
 * @Modify:
 **/
@Service
@Slf4j
public class MesShippingLoadingCheckService implements IMesShippingLoadingCheckService {

    @Autowired
    private MesLoadingVehiclesOrderRepository vehiclesOrderRepository;
    @Autowired
    private MesLoadingVehiclesOrderDetailRepository vehiclesOrderDetailRepository;

    @Autowired
    private MesShippingOrderManagementRepository shippingOrderManagementRepository;
    @Autowired
    private MesShippingOrderManagementDetailRepository shippingOrderManagementDetailRepository;
    @Autowired
    private MesProduceSnRepository produceSnRepository;
    @Autowired
    private IMesJisShippingService jisShippingService;
    @Autowired
    private MesJisShippingRepository jisShippingRepository;
    @Resource(name = MesPcnConstWords.REDIS_RES_PCN)
    private ImppRedis redisMesPcn;

    @Override
    public MesShippingLoadingCheckModel queryLoadingOrderNo(MesLoadingVehiclesOrder loadingVehiclesOrder) {

        //校验装车单是否存在
        DdlPackBean orderPackBean = DdlPackBean.getDdlPackBean(loadingVehiclesOrder.organizeCode);
        DdlPreparedPack.getStringEqualPack(loadingVehiclesOrder.getLoadingOrderNo(), "loadingOrderNo", orderPackBean);
        MesLoadingVehiclesOrder vehiclesOrder = vehiclesOrderRepository.getByProperty(orderPackBean);
        if (StringUtils.isEmpty(vehiclesOrder)) {
            throw new ImppBusiException(String.format("【%s】此装车单不存在，请检查数据！", loadingVehiclesOrder.getLoadingOrderNo()));
        }

        //校验装车单状态 必须是未装车条码
        if (vehiclesOrder.getStatus() != MesExtEnumUtil.SHIPPING_ORDER_STATUS.CREATE.getValue() && vehiclesOrder.getStatus() != MesExtEnumUtil.SHIPPING_ORDER_STATUS.PUBLISHED.getValue()) {
            throw new ImppBusiException(String.format("【%s】此装车单状态为【%s】,请选择状态为【%s】状态的装车单", loadingVehiclesOrder.getLoadingOrderNo(),
                    MesExtEnumUtil.SHIPPING_ORDER_STATUS.valueOfDescription(vehiclesOrder.getStatus()), MesExtEnumUtil.SHIPPING_ORDER_STATUS.CREATE.getDescription()));
        }


        DdlPackBean orderDetailPackBean = DdlPackBean.getDdlPackBean(loadingVehiclesOrder.organizeCode);
        DdlPreparedPack.getStringEqualPack(loadingVehiclesOrder.getLoadingOrderNo(), "loadingOrderNo", orderDetailPackBean);
        List<MesLoadingVehiclesOrderDetail> detailList = vehiclesOrderDetailRepository.findByHqlWhere(orderDetailPackBean);

        //根据扫描的装车单找到是否先装车配置
        //若配置为否，则查询装车单中所有的发运单是否都进行【排序发运校验】完成，若【排序发运校验】没完成，则提示，先进行排序发运校验。
        if (vehiclesOrder.getIsFirstInstall() == MesCommonConstant.FALSE_INTEGER) {

            //排序发运校验
            List<String> shippingOrderNoList = detailList.stream().map(k -> k.getShippingOrderNo()).collect(Collectors.toList());

            DdlPackBean orderManagementPackBean = DdlPackBean.getDdlPackBean(loadingVehiclesOrder.organizeCode);
            DdlPreparedPack.getInPackList(shippingOrderNoList, "shippingOrderNo", orderManagementPackBean);
            List<MesShippingOrderManagement> orderManagementList = shippingOrderManagementRepository.findByHqlWhere(orderManagementPackBean);
            //未校验完成的发运单
            List<String> orderNoList =
                    orderManagementList.stream().filter(k -> k.getStatus() != MesExtEnumUtil.SHIPPING_ORDER_STATUS.VERIFICATION_COMPLETED.getValue()).map(k -> k.getShippingGroupNo()).collect(Collectors.toList());

            if (!CollectionUtils.isEmpty(orderNoList)) {
                throw new ImppBusiException(String.format("【%s】此装车单中发运单为【%s】，还未做完【排序发运校验】，请检查数据", vehiclesOrder.getLoadingOrderNo(), orderNoList));
            }

        }

        MesShippingLoadingCheckModel model = new MesShippingLoadingCheckModel();
        //根据是否启用记忆
        //若启用，则获取redis数据，redis中数据为空，则获取装车单明细数据 若redis中数据不为空，则根据装车单明细中发运单号，将扫描过的标记。 若不启动则获取装车单明细数据
        if (vehiclesOrder.getIsEnableMemory() == MesCommonConstant.TRUE_INTEGER) {

            StringBuilder stringBuilder = new StringBuilder();
            stringBuilder.append(loadingVehiclesOrder.getOrganizeCode()).append(":").append(loadingVehiclesOrder.getLoadingOrderNo()).append(":");

            //获取redis数据
            if (redisMesPcn.hasKey(stringBuilder.toString())) {
                //redis中数据不为空
                List<MesLoadingVehiclesOrderDetail> isScanDetailList = (List<MesLoadingVehiclesOrderDetail>) redisMesPcn.getObject(stringBuilder.toString());

                AtomicReference<Integer> isScanCount = new AtomicReference<>(0);
                //根据装车单明细中发运单号，将扫描过的标记
                detailList.forEach(k -> {
                    Optional optional = isScanDetailList.stream().filter(scanDetail -> scanDetail.getLoadingOrderNo().equals(k.getLoadingOrderNo())).findFirst();
                    if (optional.isPresent()) {
                        k.setTempStatus(MesCommonConstant.TRUE_INTEGER);
                        isScanCount.getAndSet(isScanCount.get() + 1);
                    }
                });

                model.setIsScanCount(isScanCount.get());
            }

        }

        //查询最近扫描完成的装车单号
        DdlPackBean recentlyOrderPackBean = DdlPackBean.getDdlPackBean(loadingVehiclesOrder.organizeCode);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.SHIPPING_ORDER_STATUS.SHIPPED.getValue(), "status", recentlyOrderPackBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"modifyDatetime"}, recentlyOrderPackBean);
        MesLoadingVehiclesOrder recentlyVehiclesOrder = vehiclesOrderRepository.getByProperty(recentlyOrderPackBean);

        if (!StringUtils.isEmpty(recentlyVehiclesOrder)) {

            model.setRecentlyScanVehiclesOrder(recentlyVehiclesOrder.getLoadingOrderNo());
        }
        model.setVehiclesOrderDetailList(detailList);

        return model;
    }

    @Override
    public MesLoadingVehiclesOrder queryLoadingOrderNoByAutoPushOrder(String shippingGroupNo, String org) {


        //查询最近扫描完成的装车单号
        DdlPackBean orderPackBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.SHIPPING_ORDER_STATUS.CREATE.getValue(), "status", orderPackBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"seq"}, orderPackBean);
        MesLoadingVehiclesOrder loadingVehiclesOrder = vehiclesOrderRepository.getByProperty(orderPackBean);

        return loadingVehiclesOrder;
    }

    @Override
    public MesShippingLoadingCheckModel saveVehiclesOrderAndDetail(MesShippingLoadingCheckModel model, String org) {


        //校验发运单是否存在
        DdlPackBean orderDetailPackBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getStringEqualPack(model.getShippingOrderNo(), "shippingOrderNo", orderDetailPackBean);
        MesLoadingVehiclesOrderDetail vehiclesOrderDetail = vehiclesOrderDetailRepository.getByProperty(orderDetailPackBean);
        if (StringUtils.isEmpty(vehiclesOrderDetail)) {
            throw new ImppBusiException(String.format("【%s】此发运单不存在或还未生成装车单明细，请检查数据！", model.getShippingOrderNo()));
        }

        //根据扫描的发运单以及前端传的发运单明细，进行匹配，若没有找到扫描的发运单，则提示扫描的发运单号不在此装车单中，请检查数据！。
        Boolean scanflg = false;
        for (MesLoadingVehiclesOrderDetail mesLoadingVehiclesOrderDetail : model.getVehiclesOrderDetailList()) {
            if (mesLoadingVehiclesOrderDetail.getShippingOrderNo().equals(model.getShippingOrderNo())) {

                if (mesLoadingVehiclesOrderDetail.getTempStatus() == MesCommonConstant.TRUE_INTEGER) {
                    throw new ImppBusiException(String.format("【%s】此发运单已经扫描过，请检查数据！", model.getShippingOrderNo()));
                }
                mesLoadingVehiclesOrderDetail.setTempStatus(MesCommonConstant.TRUE_INTEGER);
                mesLoadingVehiclesOrderDetail.setTempStatusVal("扫描完成");
                scanflg = true;
            }
        }

        if (!scanflg) {
            throw new ImppBusiException(String.format("【%s】扫描的发运单号不在【%s】此装车单中，请检查数据！", model.getShippingOrderNo(), model.getLoadingOrderNo()));
        }

        //校验装车单是否存在
        DdlPackBean orderPackBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getStringEqualPack(model.getLoadingOrderNo(), "loadingOrderNo", orderPackBean);
        MesLoadingVehiclesOrder vehiclesOrder = vehiclesOrderRepository.getByProperty(orderPackBean);
        if (StringUtils.isEmpty(vehiclesOrder)) {
            throw new ImppBusiException(String.format("【%s】此装车单不存在，请检查数据！", model.getLoadingOrderNo()));
        }
        if (vehiclesOrder.getIsEnableMemory() == MesCommonConstant.TRUE_INTEGER) {

            StringBuilder stringBuilder = new StringBuilder();
            stringBuilder.append(org).append(":").append(model.getLoadingOrderNo()).append(":");

            redisMesPcn.putHash(stringBuilder.toString(), MesPcnConstWords.BIND_KEY_PART, model.getVehiclesOrderDetailList(), -1);
        }

        Long count = model.getVehiclesOrderDetailList().stream().filter(k -> k.getTempStatus() == MesCommonConstant.TRUE_INTEGER).count();
        model.setIsScanCount(count.intValue());

        //全部扫描完成
        if (count.intValue() == model.getVehiclesOrderDetailList().size()) {

            //判断是否先装车
            if (vehiclesOrder.getIsFirstInstall() == MesCommonConstant.TRUE_INTEGER) {

                //判断是先装车
                //修改装车单 装车单明细数据状态。

                //装车单主表
                vehiclesOrder.setStatus(MesExtEnumUtil.LOADING_ORDER_STATUS.LOADED.getValue());

                //装车单明细
                DdlPackBean vehiclesOrderDetailPackBean = DdlPackBean.getDdlPackBean(org);
                DdlPreparedPack.getStringEqualPack(vehiclesOrder.getLoadingOrderNo(), "loadingOrderNo", vehiclesOrderDetailPackBean);
                List<MesLoadingVehiclesOrderDetail> vehiclesOrderDetailList = vehiclesOrderDetailRepository.findByHqlWhere(vehiclesOrderDetailPackBean);
                vehiclesOrderDetailList.forEach(k -> k.setStatus(MesExtEnumUtil.LOADING_DETAIL_ORDER_STATUS.LOADED.getValue()));

                vehiclesOrderDetailRepository.saveAll(vehiclesOrderDetailList);
                vehiclesOrderRepository.save(vehiclesOrder);

            } else {

                //判断否先装车
                //修改对应装车单 装车单明细 发运单 发运明细 条码数据 报工给wms。

                //排序发运校验
                List<String> shippingOrderNoList = model.getVehiclesOrderDetailList().stream().map(k -> k.getShippingOrderNo()).collect(Collectors.toList());

                //发运单
                DdlPackBean orderManagementPackBean = DdlPackBean.getDdlPackBean(org);
                DdlPreparedPack.getInPackList(shippingOrderNoList, "shippingOrderNo", orderManagementPackBean);
                List<MesShippingOrderManagement> orderManagementList = shippingOrderManagementRepository.findByHqlWhere(orderManagementPackBean);

                //发运单明细
                List<MesShippingOrderManagementDetail> orderManagementDetailList = shippingOrderManagementDetailRepository.findByHqlWhere(orderManagementPackBean);

                //条码
                List<String> snList = orderManagementDetailList.stream().map(k -> k.getBarcode()).collect(Collectors.toList());
                DdlPackBean producePackBean = DdlPackBean.getDdlPackBean(org);
                DdlPreparedPack.getInPackList(snList, "serialNumber", producePackBean);
                List<MesProduceSn> produceSnList = produceSnRepository.findByHqlWhere(producePackBean);

                //装车单明细
                DdlPackBean vehiclesOrderDetailPackBean = DdlPackBean.getDdlPackBean(org);
                DdlPreparedPack.getStringEqualPack(vehiclesOrder.getLoadingOrderNo(), "loadingOrderNo", vehiclesOrderDetailPackBean);
                List<MesLoadingVehiclesOrderDetail> vehiclesOrderDetailList = vehiclesOrderDetailRepository.findByHqlWhere(vehiclesOrderDetailPackBean);

                orderManagementList.forEach(k -> k.setStatus(MesExtEnumUtil.SHIPPING_ORDER_STATUS.SHIPPED.getValue()));
                orderManagementDetailList.forEach(k -> k.setStatus(MesExtEnumUtil.SHIPPING_ORDER_STATUS.SHIPPED.getValue()));
                produceSnList.forEach(k -> k.setSnStatus(MesExtEnumUtil.PRODUCE_SN_STATUS.SHIPPED.getValue()));
                vehiclesOrder.setStatus(MesExtEnumUtil.LOADING_ORDER_STATUS.SHIPPING.getValue());
                vehiclesOrderDetailList.forEach(k -> {
                    k.setStatus(MesExtEnumUtil.LOADING_DETAIL_ORDER_STATUS.SHIPPING.getValue());
                    k.setShippingStatus(MesExtEnumUtil.LOADING_ORDER_DETAIL_SHIPPING_STATUS.SHIPPINGED.getValue());
                });

                List<MesJisShipping> jisShippingList = jisShippingService.getJisShipping(orderManagementDetailList, org);

                //保存当前发运单明细报工
                jisShippingRepository.saveAll(jisShippingList);
                shippingOrderManagementRepository.saveAll(orderManagementList);
                shippingOrderManagementDetailRepository.saveAll(orderManagementDetailList);
                produceSnRepository.saveAll(produceSnList);
                vehiclesOrderDetailRepository.saveAll(vehiclesOrderDetailList);
                vehiclesOrderRepository.save(vehiclesOrder);

            }

        }

        return model;
    }
}
